@@ -11,17 +11,17 @@ logger = getLogger(__name__)
 
 class Recipe(PackagingRecipe):
     def install(self):
-        self.delete_non_production_packages_from_cache_dist()
-        self.write_buildout_configuration_file_for_production()
-        utils.compiler.compile_binary_distributions(self.get_buildout_dir(),
-                                                    self.get_download_cache_dist(),
-                                                    self.get_eggs_directory())
-        utils.download_buildout(self.get_download_cache_dist())
-        utils.download_distribute(self.get_download_cache_dist())
-        utils.download_setuptools(self.get_download_cache_dist())
-        package = self.build_package()
-        logger.debug("Built {}".format(package))
-        return [package, ]
+        with self.with_most_mortem():
+            self.delete_non_production_packages_from_cache_dist()
+            self.write_buildout_configuration_file_for_production()
+            utils.compiler.compile_binary_distributions(self.get_buildout_dir(),
+                                                        self.get_download_cache_dist(),
+                                                        self.get_eggs_directory())
+            utils.download_buildout(self.get_download_cache_dist())
+            utils.download_setuptools(self.get_download_cache_dist())
+            package = self.build_package()
+            logger.debug("Built {}".format(package))
+            return [package, ]
 
     def build_package(self):
         self._directories = set()
