@@ -13,6 +13,8 @@ logger = getLogger(__name__)
 BYPASS_CUSTOM_ACTION_PROPERTY = "NO_CUSTOM_ACTIONS"
 CONDITION_DURING_INSTALL_OR_REPAIR = 'NOT Installed OR MaintenanceMode="Modify"'
 CONDITION_DURING_UNINSTALL_NOT_UPGRADE = 'REMOVE="ALL" AND NOT UPGRADINGPRODUCTCODE'
+CONDITION_DURING_UPGRADE = "UPGRADINGPRODUCTCODE"
+CONDITION_DURING_UPGRADE_AND_UNINSTALL = "({}) OR ({})".format(CONDITION_DURING_UPGRADE, CONDITION_DURING_UNINSTALL_NOT_UPGRADE)
 DowngradeErrorMessage = 'A later version of [ProductName] is already installed. Setup will now exit.'
 LAUNCH_CONDITION__WINDOWS_2008_AND_R2_ONLY = \
     "(Not Version9X) And (Not VersionNT=400) And (Not VersionNT=500) And (Not VersionNT=501) And" + \
@@ -22,6 +24,7 @@ OS_REQUIREMENTS_LAUNCH_CONDITION_MESSAGE = "The operating system is not adequate
 OS_32BIT_ON_64BIT_LAUNCH_CONDITION_MESSAGE = "[ProductName] installation on a 64-bit operating system requires the 64-bit installation package. Please get the 64-bit package and try again."
 OPERATING_SYSTEMS = {
                      # OS Name: (allow-install, condition)
+                     "Windows 8.1": (False, '(VersionNT=603 And (MsiNTProductType=1))'),
                      "Windows Server 2012 R2": (True, '(VersionNT=603 And (MsiNTProductType=2 Or MsiNTProductType=3))'),
                      "Windows Server 2012": (True, '(VersionNT=602 And (MsiNTProductType=2 Or MsiNTProductType=3))'),
                      "Windows 8": (False, '(VersionNT=602 And (MsiNTProductType=1))'),
@@ -42,22 +45,22 @@ SILENT_LAUNCHER = "silent_launcher-{}.exe"
 
 class Recipe(PackagingRecipe):
     def install(self):
-        self.delete_non_production_packages_from_cache_dist()
-        self.signtool = self.get_signtool()
-        self.write_buildout_configuration_file_for_production()
-        utils.download_buildout(self.get_download_cache_dist())
-        utils.download_distribute(self.get_download_cache_dist())
-        utils.download_setuptools(self.get_download_cache_dist())
-        silent_launcher = self.get_silent_launcher()
-        if self.should_sign_files():
-            self.sign_all_executables_in_project()
-            self.signtool.sign_file(silent_launcher)
-        package = self.build_package(silent_launcher)
-        logger.debug("Built {}".format(package))
-        if self.should_sign_files():
-            self.signtool.sign_file(package)
-            logger.debug("Signed {}".format(package))
-        return [package, ]
+        with self.with_most_mortem():
+            self.delete_non_production_packages_from_cache_dist()
+            self.signtool = self.get_signtool()
+            self.write_buildout_configuration_file_for_production()
+            utils.download_buildout(self.get_download_cache_dist())
+            utils.download_setuptools(self.get_download_cache_dist())
+            silent_launcher = self.get_silent_launcher()
+            if self.should_sign_files():
+                self.sign_all_executables_in_project()
+                self.signtool.sign_file(silent_launcher)
+            package = self.build_package(silent_launcher)
+            logger.debug("Built {}".format(package))
+            if self.should_sign_files():
+                self.signtool.sign_file(package)
+                logger.debug("Signed {}".format(package))
+            return [package, ]
 
     def get_silent_launcher(self):
         from tempfile import mkstemp
@@ -107,7 +110,7 @@ class Recipe(PackagingRecipe):
         from .wix import Wix
         wix = Wix(self.get_product_name(), self.get_project_version__short(),
                   self.get_platform_arch(), self.get_upgrade_code(), self.get_description(),
-                  self.get_company_name())
+                  self.get_company_name(), self.get_documentation_url())
         silent_launcher_file_id = self._put_all_files(wix, silent_launcher)
         self._append_bindir_to_system_path(wix)
         self._append_custom_actions(wix, silent_launcher_file_id)
@@ -126,6 +129,10 @@ class Recipe(PackagingRecipe):
         if dialog_bmp:
             logger.info("Setting custom dialog {}".format(dialog_bmp))
             wix.new_element("WixVariable", {"Id": "WixUIDialogBmp", "Value": dialog_bmp}, wix.product)
+        eula = self.get_eula_rtf()
+        if eula:
+            logger.info("Setting custom eula {}".format(eula))
+            wix.new_element("WixVariable", {"Id": "WixUILicenseRtf", "Value": eula}, wix.product)
         return wix
 
     @contextmanager
@@ -176,28 +183,58 @@ class Recipe(PackagingRecipe):
         commandline = r'''"[INSTALLDIR]parts\python\bin\python.exe" "-c" "from shutil import rmtree; rmtree('[INSTALLDIR]eggs', True)"'''
         action = wix.add_deferred_in_system_context_custom_action('os_removedirs_eggs', commandline,
                                                                   condition=CONDITION_DURING_INSTALL_OR_REPAIR,
-                                                                  silent_launcher_file_id=silent_launcher_file_id)
+                                                                  silent_launcher_file_id=silent_launcher_file_id,
+                                                                  text="Removing temporary files, this may take a few minutes")
         return action.get('Id')
 
     def _append_bootstrap_custom_action(self, wix, os_removedirs_eggs_id, silent_launcher_file_id):
         commandline = r'"[INSTALLDIR]parts\python\bin\python.exe" bootstrap.py ' + \
-                      r'--download-base=.cache\dist --setup-source=.cache\dist\ez_setup.py'
+                      r'--download-base="[INSTALLDIR].cache\dist" ' + \
+                      r'--setup-source="[INSTALLDIR].cache\dist\ez_setup.py'
         action = wix.add_deferred_in_system_context_custom_action('bootstrap', commandline,
                                                                   after=os_removedirs_eggs_id,
                                                                   condition=CONDITION_DURING_INSTALL_OR_REPAIR,
-                                                                  silent_launcher_file_id=silent_launcher_file_id)
+                                                                  silent_launcher_file_id=silent_launcher_file_id,
+                                                                  text="Bootstrapping, this may take a few minutes")
+
         return action.get('Id')
 
     def _append_buildout_custom_action(self, wix, bootstrap_id, silent_launcher_file_id):
-        action = wix.add_deferred_in_system_context_custom_action('buildout', r'"[INSTALLDIR]bin\buildout.exe"',
+        commandline = r'"[INSTALLDIR]bin\buildout.exe" -U -c "[INSTALLDIR]buildout.cfg"'
+        action = wix.add_deferred_in_system_context_custom_action('buildout', commandline,
                                                                   after=bootstrap_id,
                                                                   condition=CONDITION_DURING_INSTALL_OR_REPAIR,
-                                                                  silent_launcher_file_id=silent_launcher_file_id)
+                                                                  silent_launcher_file_id=silent_launcher_file_id,
+                                                                  text="Extracting dependencies, this may take a few minutes")
+
+
+    def _append_close_application_action(self, wix, bootstrap_id, silent_launcher_file_id):
+        commandline = r'"[INSTALLDIR]bin\buildout.exe" -U -c "[INSTALLDIR]buildout.cfg" install debug-logging close-application'
+        condition = CONDITION_DURING_UPGRADE_AND_UNINSTALL
+        action = wix.add_deferred_in_system_context_custom_action('close_application', commandline,
+                                                                  after='InstallInitialize',
+                                                                  condition=condition,
+                                                                  silent_launcher_file_id=silent_launcher_file_id,
+                                                                  text="Closing open applications, this may take a few minutes")
 
     def _append_custom_actions(self, wix, silent_launcher_file_id):
+        recipe = self.get_recipe_section()
+        existing_installdir = recipe.get("existing-installdir")
+        # http://stackoverflow.com/questions/12576807/how-to-set-targetdir-or-installdir-from-a-registry-entry
+        if existing_installdir:
+            root, key = existing_installdir.split("\\", 1) # HKLM, SOFTWARE\...\InstallPath
+            key, value = key.rsplit("\\", 1) # Software...\, InstallPath
+            wix.add_existing_install_dir(root, key, value)
+            wix.set_custom_install_dir(recipe.get("custom-installdir", "[EXISTINGINSTALLDIR]"))
+            wix.prevent_user_from_choosing_installation_directory()
+        elif recipe.get("custom-installdir"):
+            wix.set_custom_install_dir(recipe.get("custom-installdir"))
+            wix.prevent_user_from_choosing_installation_directory()
+
         os_removedirs_eggs_id = self._append_os_removedirs_eggs(wix, silent_launcher_file_id)
         bootstrap_id = self._append_bootstrap_custom_action(wix, os_removedirs_eggs_id, silent_launcher_file_id)
         self._append_buildout_custom_action(wix, bootstrap_id, silent_launcher_file_id)
+        self._append_close_application_action(wix, bootstrap_id, silent_launcher_file_id)
         wix.new_element("Property", {"Id": BYPASS_CUSTOM_ACTION_PROPERTY, "Value":"0"}, wix.product)
 
     def _add_launch_conditions(self, wix):
@@ -237,17 +274,22 @@ class Recipe(PackagingRecipe):
 
     def _add_project_entry_points(self, wix, silent_launcher_file_id):
         for key, value in {'post_install': {'after': 'custom_action_buildout',
+                                            'before': None,
                                             'condition': CONDITION_DURING_INSTALL_OR_REPAIR},
-                           'pre_uninstall': {'after': 'InstallInitialize',
+                           'pre_uninstall': {'after': None,
+                                             'before': 'custom_action_close_application',
                                              'condition': CONDITION_DURING_UNINSTALL_NOT_UPGRADE}, }.items():
             script_name = self.get_script_name(key)
-            if script_name in ['', None]:
+            if script_name in ['', 'None', None]:
                 continue
             args = self.get_script_args(key)
             commandline = r'"[INSTALLDIR]\bin\{}.exe" {}'.format(script_name, args)
-            wix.add_deferred_in_system_context_custom_action(script_name, commandline, after=value['after'],
+            text = "Running {} actions, this may take a few minutes".format(key.replace('_', ' '))
+            wix.add_deferred_in_system_context_custom_action(script_name, commandline,
+                                                             after=value['after'], before=value['before'],
                                                              condition=value['condition'],
-                                                             silent_launcher_file_id=silent_launcher_file_id)
+                                                             silent_launcher_file_id=silent_launcher_file_id,
+                                                             text=text)
 
     def _add_shortcuts(self, wix):
         if not self.get_startmenu_shortcuts() or not self.get_shortcuts_icon():
